# frozen_string_literal: true

require_relative 'client/version'
require_relative 'client/configuration'
require_relative 'client/error'

module Dipps
  module Client
    SCHEMA_PATH = File.expand_path(
      File.join(File.dirname(__FILE__), '..', '..', 'vendor', 'schema.json')
    ).freeze

    class << self
      def configure
        yield(configuration)

        unless require_relative 'client/queries'
          raise 'Dipps::Client has already been configured.'
        end
      end

      def configuration
        @configuration ||= Dipps::Client::Configuration.new
      end

      def perform_query(query:, result_field:, variables: {})
        result = GraphqlClient.query(query, variables: variables)

        if result.errors.any?
          raise(Dipps::Client::Error, result.errors.details.inspect)
        end

        result.data.to_h.fetch(result_field)
      end

      def create_own_invoice(freelance_profile_id:, attributes:)
        perform_query(
          query: Dipps::Client::Queries::CreateOwnInvoice,
          result_field: 'createOwnInvoice',
          variables: {
            freelance_profile_id: freelance_profile_id,
            attributes: attributes
          }
        )
      end

      def clients(org_number: nil, email: nil)
        perform_query(
          query: Dipps::Client::Queries::Clients,
          result_field: 'clients',
          variables: {
            orgNumber: org_number,
            email: email
          }
        )
      end

      def create_client(attributes:)
        perform_query(
          query: Dipps::Client::Queries::CreateClient,
          result_field: 'createClient',
          variables: {
            attributes: attributes
          }
        )
      end

      def update_client(id:, attributes:)
        perform_query(
          query: Dipps::Client::Queries::UpdateClient,
          result_field: 'updateClient',
          variables: {
            id: id,
            attributes: attributes
          }
        )
      end
    end
  end
end
